#include "selenite/palette.h"
#include "json.hpp"
#include <fstream>
#include <optional>
#include <string>

namespace selenite
{
Color hexStringToColor(const std::string &hexString)
{
	int r, g, b;
	std::sscanf(hexString.c_str(), "#%02x%02x%02x", &r, &g, &b);
	return {
		.r = r,
		.g = g,
		.b = b,
		.hex = hexString,
	};
}

std::optional<Palette> parseJson(const std::string &path)
{
	std::ifstream file(path);
	if (!file) {
		return std::nullopt;
	}
	auto json = nlohmann::json::parse(file);

	if (!json.is_object() || !json["primary"].is_string() || !json["secondary"].is_string() ||
		!json["background"].is_string() || !json["background2"].is_string() ||
		!json["background4"].is_string() || !json["background3"].is_string() ||
		!json["pane"].is_string() || !json["foreground"].is_string() ||
		!json["foregroundHint"].is_string() || !json["green"].is_string() ||
		!json["orange"].is_string() || !json["red"].is_string()) {
		return std::nullopt;
	}

	Palette palette{
		.primary = hexStringToColor(json["primary"].get<std::string>()),
		.secondary = hexStringToColor(json["secondary"].get<std::string>()),
		.background = hexStringToColor(json["background"].get<std::string>()),
		.background2 = hexStringToColor(json["background2"].get<std::string>()),
		.background3 = hexStringToColor(json["background3"].get<std::string>()),
		.background4 = hexStringToColor(json["background4"].get<std::string>()),
		.pane = hexStringToColor(json["pane"].get<std::string>()),
		.foreground = hexStringToColor(json["foreground"].get<std::string>()),
		.foregroundHint = hexStringToColor(json["foregroundHint"].get<std::string>()),
		.green = hexStringToColor(json["green"].get<std::string>()),
		.orange = hexStringToColor(json["orange"].get<std::string>()),
		.red = hexStringToColor(json["red"].get<std::string>()),
	};
	return palette;
}
} // namespace selenite
